import { StrictMode } from 'react';
import { createRoot } from 'react-dom/client';
import App from './App.tsx';
import './index.css';
import '@uiw/react-md-editor/markdown-editor.css';
import '@uiw/react-markdown-preview/markdown.css';

// Import event bus service to set it up early
import { eventBusService } from './utils/eventBusService';
import { initEventBus } from './utils/eventBusService';
import { eventBus } from './utils/eventBus';

// Initialize the event bus service with the actual implementation
initEventBus(eventBus);

// Import auth debug tools and expose them globally for console access
import * as AuthDebug from './utils/auth-debug';
// @ts-ignore - Add to window for debugging
window.AuthDebug = AuthDebug;
console.log('Auth debug tools available in console as AuthDebug. Call AuthDebug.debugAuthState() for diagnostics.');

// Add React Query debug function
const debugReactQuery = () => {
  console.group('React Query Debug Info');
  console.log('__REACT_QUERY_GLOBAL_CLIENT__ exists:', typeof window.__REACT_QUERY_GLOBAL_CLIENT__ !== 'undefined');
  console.log('_reactQueryClient exists:', typeof window._reactQueryClient !== 'undefined');
  
  if (window.__REACT_QUERY_GLOBAL_CLIENT__) {
    console.log('Has invalidateQueries:', typeof window.__REACT_QUERY_GLOBAL_CLIENT__.invalidateQueries === 'function');
    console.log('Has setQueryData:', typeof window.__REACT_QUERY_GLOBAL_CLIENT__.setQueryData === 'function');
    console.log('Has getQueryData:', typeof window.__REACT_QUERY_GLOBAL_CLIENT__.getQueryData === 'function');
  }
  console.groupEnd();
  return 'React Query debug info logged to console';
};

// @ts-ignore - Add to window for debugging
window.debugReactQuery = debugReactQuery;
console.log('React Query debug tools available in console. Call debugReactQuery() to check React Query setup.');

// Use a function to create the root to allow for potential error handling
const renderApp = async () => {
  // Initialize auth service
  try {
    // Initialize the centralized auth service from the new location
    const { initialize } = await import('./auth');
    const initialized = await initialize();
    console.log('Centralized auth service initialized:', initialized);
  } catch (error) {
    console.error('Failed to initialize auth service:', error);
    // Continue rendering the app, it will handle auth initialization failures
  }
  
  const container = document.getElementById('root');
  
  if (!container) {
    console.error('Root element not found');
    return;
  }
  
  const root = createRoot(container);
  
  // Wrap in error boundary for production
  root.render(
    <StrictMode>
      <App />
    </StrictMode>
  );
};

// Setup dynamic import error handler to deal with cache issues
const setupImportErrorHandler = () => {
  try {
    // Track reload attempts to prevent endless refresh loops
    const RELOAD_KEY = 'last_reload_timestamp';
    const RELOAD_THRESHOLD_MS = 10000; // 10 seconds
    const RELOAD_COUNT_KEY = 'reload_count';
    const MAX_RELOADS = 2; // Maximum number of reloads to attempt
    
    // Check if we've already reloaded recently
    const hasReloadedRecently = () => {
      try {
        const lastReload = parseInt(sessionStorage.getItem(RELOAD_KEY) || '0');
        const reloadCount = parseInt(sessionStorage.getItem(RELOAD_COUNT_KEY) || '0');
        const now = Date.now();
        
        // If we've reloaded too recently or too many times, don't reload again
        if ((now - lastReload < RELOAD_THRESHOLD_MS) || reloadCount >= MAX_RELOADS) {
          console.warn(`Skipping reload: ${reloadCount} reloads in the last ${Math.round((now - lastReload)/1000)}s`);
          return true;
        }
        
        return false;
      } catch {
        return false;
      }
    };
    
    // Track a reload attempt
    const trackReload = () => {
      try {
        const now = Date.now();
        const currentCount = parseInt(sessionStorage.getItem(RELOAD_COUNT_KEY) || '0');
        sessionStorage.setItem(RELOAD_KEY, now.toString());
        sessionStorage.setItem(RELOAD_COUNT_KEY, (currentCount + 1).toString());
      } catch {
        // Ignore errors in tracking
      }
    };
    
    // Safe reload function that prevents loops
    const safeReload = () => {
      if (!hasReloadedRecently()) {
        // Track this reload
        trackReload();
        
        // Use a gentler reload method that keeps user's position
        console.warn('Reloading page to fix module loading issues...');
        window.location.reload();
      } else {
        console.warn('Suppressing reload to prevent refresh loop');
      }
    };
    
    // Only handle severe errors that are definitely module loading related
    // We're being much more selective about which errors trigger a reload
    window.addEventListener('error', (event) => {
      const errorMessage = String(event.message || '') + String(event?.error?.message || '');
      
      // Only reload for fatal module loading failures
      if (errorMessage.includes('Failed to fetch dynamically imported module') && 
          errorMessage.includes('404')) {
        console.warn('Detected missing module error, attempting reload...');
        safeReload();
      }
    });

    // Gentler handling of unhandled promise rejections
    window.addEventListener('unhandledrejection', (event) => {
      const errorMessage = String(event.reason || '');
      
      // Only reload for specific critical module loading errors
      if (errorMessage.includes('Failed to fetch dynamically imported module') && 
          errorMessage.includes('404')) {
        console.warn('Detected missing module promise rejection, attempting reload...');
        safeReload();
      }
    });
    
    // Mark this as a modern browser for Vite
    window.__vite_is_modern_browser = true;
    
    // Add version info to localStorage to track version changes
    try {
      const lastVersion = localStorage.getItem('app_version') || '';
      const currentVersion = import.meta.env.VITE_APP_VERSION || '2025-04-12-3';
      
      if (lastVersion !== currentVersion) {
        console.log('New app version detected:', currentVersion);
        
        // Only clear session storage on first load, not during navigation
        if (!sessionStorage.getItem('version_initialized')) {
          console.log('Initializing new version, clearing session storage');
          // Mark that we've initialized this version
          sessionStorage.setItem('version_initialized', 'true');
          // Clear session storage which might contain stale module paths
          sessionStorage.clear();
          sessionStorage.setItem('version_initialized', 'true');
        }
        
        // Update stored version
        localStorage.setItem('app_version', currentVersion);
      }
    } catch (storageError) {
      console.error('Error managing version storage:', storageError);
    }
  } catch (e) {
    console.error('Error setting up module cache handling:', e);
  }
};

// Initialize the error handler
setupImportErrorHandler();

// Add some basic performance measurement
const startTime = performance.now();

// Initialize the app
renderApp().catch(err => {
  console.error('App initialization error:', err);
});

// Log performance metrics in development
if (import.meta.env.DEV) {
  window.addEventListener('load', () => {
    const endTime = performance.now();
    console.log(`App initialization time: ${(endTime - startTime).toFixed(2)}ms`);
  });
}